import {
  Metadata, IEventParam, Loader, IDictionary, Config, dayjs as moment, Http,
} from '@zeedhi/core';

import {
  GridEditable, Modal, ModalService, Form, Button, DialogService, Select, Grid,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Ger04000NutrientesPorProdutoController {
  public gridEditableController = Loader.getInstance('GridEditableController');
  
  private utilities = Loader.getInstance('Utilities');
  
  public modalController = Loader.getInstance('ModalController');
  
  public reportController = Loader.getInstance('ReportController');
  
  public crudController = Loader.getInstance('CrudController');
  
  private modalAddNutri!: Modal;
  
  private modalRelatorioNutriProd!: Modal;

  private modalExport!: Modal;
  
  public disablePropChange = false;

  public selectedLine: IDictionary = {};
  
  public filterNutrientes() {
    const produto = Metadata.getInstance<Select>('CDPRODUTO').value;
    const nutriente: GridEditable = Metadata.getInstance('NUTRIENTE');
    const form = Metadata.getInstance<Form>('formFilter').value;
    if (form.CDPRODUTO !== null) {
      const conditions = [
        {
          name: 'CDPRODUTO',
          operator: '=',
          value: produto.CDPRODUTO,
        },
      ];
      nutriente.datasource.filter = conditions;
      nutriente.datasource.get();
      nutriente.selectedRows = [];
    } else {
      nutriente.datasource.data = [];
      nutriente.selectedRows = [];
    }
  }
  
  public addNutriente() {
    const produto = Metadata.getInstance<Select>('CDPRODUTO').value;
    if (this.utilities.centralIsValid() && produto) {
      if (!this.modalAddNutri) {
        this.createModalAddNutrientes().then(() => {
          this.modalAddNutri.show();
          this.filterDataSourceAddNutri();
        });
      } else {
        this.modalAddNutri.show();
      }
    }
  }
  
  private async createModalAddNutrientes() {
    this.modalAddNutri = await ModalService.createFromJson('planejamento/parametrizacao/modals/ger04000_nutriente', true);
  }
  
  public closeNutriModalAdd() {
    const gridAddNutri = Metadata.getInstance<GridEditable>('gridAddNutriente');
    this.gridEditableController.cancelEditing({ component: gridAddNutri });
    this.modalAddNutri.hide();
  }
  
  public filterDataSourceAddNutri() {
    const gridAddNutri = Metadata.getInstance<GridEditable>('gridAddNutriente');
    const produto = Metadata.getInstance<Select>('CDPRODUTO').value;

    const conditions = [
      {
        name: 'CDPRODUTO',
        operator: '=',
        value: produto.CDPRODUTO,
      },
    ];

    gridAddNutri.datasource.filter = conditions;
    gridAddNutri.datasource.get();
  }
  
  public saveAddNutrientes() {
    const gridGrupoNutri: GridEditable = Metadata.getInstance('gridAddNutriente');
    const editedRows: IDictionary = gridGrupoNutri.getEditedRows();
    
    if (gridGrupoNutri.selectedRows.length && editedRows.length === 0) this.utilities.showAlert('alertError', 'pme::qtdFonteVazio', 'warning');
    if (editedRows.length > 0) {
      const gridNutrientes: GridEditable = Metadata.getInstance('NUTRIENTE');
      const produto = Metadata.getInstance<Select>('CDPRODUTO').value;
      const buttonAdd = Metadata.getInstance<Button>('saveButton');
      const buttonCancel = Metadata.getInstance<Button>('cancelButton');
      let fonte;

      editedRows.forEach((row: IDictionary) => {
        row.CDPRODUTO = produto.CDPRODUTO;
        row.DTULTATUNUPR = moment().format('DD/MM/YYYY HH:mm:ss');
        if (!row.QTNUTRPROD) row.QTNUTRPROD = 0;
        if (!row.DSFONTNUTR) fonte = '';
        row.__is_new = true;
      });
      
      if (fonte === '') this.utilities.showAlert('alertError', 'pme::qtdFonteVazio', 'warning');
      else {
        this.crudController.saveRows(editedRows, 'nutrprod', gridGrupoNutri).then(() => {
          gridNutrientes.datasource.get();
          buttonAdd.disabled = true;
          buttonCancel.disabled = true;
          this.gridEditableController.reload({ component: gridNutrientes });
          this.gridEditableController.reload({ component: gridGrupoNutri });
          this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
        }).catch((error: string) => {
          this.utilities.showAlert('alert', `pme::${error}`, 'error');
        });
      }
    }
  }
  
  public async deleteCheckedRows() {
    const gridNutrientes: GridEditable = Metadata.getInstance('NUTRIENTE');
    const checkedRows = gridNutrientes.selectedRows;
    
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'confirmDeleteNutri',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteNutriente() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }
  
  public deleteNutriente() {
    const gridNutrientes: GridEditable = Metadata.getInstance('NUTRIENTE');
    const button = Metadata.getInstance<Button>('yes');
    const checkedRows = gridNutrientes.selectedRows;
    const dataSourceName = '/nutrprod';
    this.utilities.activeLoadingButton(button);
    this.crudController.deleteRows(checkedRows, dataSourceName).then(() => {
      gridNutrientes.cancelEditedRows();
      gridNutrientes.datasource.get();
      gridNutrientes.selectedRows = [];
      DialogService.hide();
      this.utilities.disableLoadingButton(button);
      
      this.utilities.showAlert('alertError', 'pme::excluidoComSucesso', 'success');
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
      DialogService.hide();
    });
  }
  
  public saveNutrientes() {
    const nutriente: GridEditable = Metadata.getInstance('NUTRIENTE');
    const editedRows: IDictionary = nutriente.getEditedRows();
    if (editedRows.length > 0) {
      const produto = Metadata.getInstance<Select>('CDPRODUTO').value;
      
      editedRows.forEach((row: IDictionary) => {
        row.CDPRODUTO = produto.CDPRODUTO;
        row.DTULTATUNUPR = moment().format('DD/MM/YYYY HH:mm:ss');
        if (!row.QTNUTRPROD) row.QTNUTRPROD = 0;
        row.__is_new = false;
      });

      this.crudController.saveRows(editedRows, 'nutrprod', nutriente).then(() => {
        nutriente.datasource.get();
        this.gridEditableController.reload({ component: nutriente });
        this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }
    
  public openModalRelatorioNutriProd() {
    if (!this.modalRelatorioNutriProd) {
      this.createModalRelatorioNutriProd().then(() => {
        this.modalRelatorioNutriProd.show();
      });
    } else {
      this.modalRelatorioNutriProd.show();
    }
  }
  
  private async createModalRelatorioNutriProd() {
    this.modalRelatorioNutriProd = await ModalService.createFromJson('planejamento/parametrizacao/modals/ger04000_relatorio_nutri_prod', true);
  }
  
  public cancelModalRelatorioNutriProd() {
    this.modalRelatorioNutriProd.hide();
  }
  
  private setParams(row: IDictionary) {
    const mask: string = (Config.masks.productMask as PmeMask).pattern;
    row.PRODUCTMASK = this.utilities.formatReportMask(mask);
    row.TAMACNIVEL = this.utilities.calcTamacNivel(row.NIVEL, 'produto');
    row.CDNVPRODUTO = row.NIVEL ? this.utilities.getReportMaskNivel(mask, row.NIVEL) : 0;
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
  }
  
  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('formRelatorioNutriProd');
    const row: IDictionary = form.value;
    
    row.reportFormat = component.name;
    this.setParams(row);
    
    this.utilities.activeLoadingButton(component);
    
    this.reportController.generateReport('report/NutriProd', row, row.CD_REPORT_CODE).finally(() => {
      this.utilities.disableLoadingButton(component);
    });
  }
  
  public setfullscreen({ component }: IEventParam<Button>) {
    this.modalAddNutri.fullscreen = !this.modalAddNutri.fullscreen;
    this.modalAddNutri.cssClass = this.modalAddNutri.fullscreen ? 'fullscreen-fill-content' : 'modal-emissao-cardapio';
    
    component.iconName = (this.modalAddNutri.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
  }

  get controlExportButton() {
    const grid = Metadata.getInstance<GridEditable>('NUTRIENTE');
    if (grid.selectedRows.length === 0) return true;
    return false;
  }

  public async createModalExport() {
    this.modalExport = await ModalService.createFromJson('planejamento/parametrizacao/modals/formExportNutrientes', true);
  }

  public openExportNutrientes() {
    const grid = Metadata.getInstance<GridEditable>('NUTRIENTE');
    if (grid.selectedRows.length !== 0) {
      this.createModalExport().then(() => {
        this.modalExport.show();
      });
    }
  }

  public filterExport({ component }: IEventParam<Select>) {
    const form = Metadata.getInstance<Form>('formFilter').value;
    const produtoOrigem = form.CDPRODUTO.CDPRODUTO;
    const formFilterExport = Metadata.getInstance<Form>('formFilterExportNutri');
    const grid: GridEditable = Metadata.getInstance('gridExportNutrientes');
    if (formFilterExport.validate()) {
      const prodIni = formFilterExport.value.PRODINICIAL.CDARVPROD;
      const prodFin = formFilterExport.value.PRODFINAL.CDARVPROD;
      this.reportController.verifyProductIni(component);
      const conditions = [
        {
          name: 'CDPRODORIGEM',
          operator: '=',
          value: produtoOrigem,
        },
        {
          name: 'CDARVPRODINI',
          operator: '=',
          value: prodIni,
        },
        {
          name: 'CDARVPRODFIN',
          operator: '=',
          value: prodFin,
        },
      ];
    
      grid.datasource.filter = conditions;
      grid.datasource.get();
    } else grid.datasource.data = [];
  }

  public setFilterProducts({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'MAXCDNVPRODUTO',
        operator: '=',
        value: 1,
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public openDialogExport() {
    const grid = Metadata.getInstance<Grid>('gridExportNutrientes');
    const form = Metadata.getInstance<Form>('formFilterExportNutri');
    this.selectedLine = grid.selectedRows;
    if (this.selectedLine.length === 0) {
      form.validate();
      this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    } else {
      DialogService.show({
        name: 'pme_warningImportacao',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::acresSobreNutrientes',
        buttons: [
          {
            name: 'acrescentar',
            component: 'ZdButton',
            label: 'pme::acrescentar',
            events: { click: () => this.acrescentar(this.selectedLine) },
            outline: true,
          },
          {
            name: 'sobrepor',
            component: 'ZdButton',
            label: 'pme::sobrepor',
            events: { click: () => this.sobrepor(this.selectedLine) },
            outline: true,
          },
          {
            name: 'cancelar',
            component: 'ZdButton',
            label: 'pme::cancelar',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }

  public acrescentar(checkedRows: IDictionary) {
    const formFilter = Metadata.getInstance<Form>('formFilter').value;
    const nutriente: GridEditable = Metadata.getInstance('NUTRIENTE');
    const rowNutriente = nutriente.selectedRows;
    const nutriSelected = rowNutriente.map((row: IDictionary) => row.CDNUTRIENTE);
    const nutrienteExport: Grid = Metadata.getInstance('gridExportNutrientes');
    const rowProdutos = nutrienteExport.selectedRows;
    const produtosSelected = rowProdutos.map((row: IDictionary) => row.CDARVPROD);
    const button = Metadata.getInstance<Button>('acrescentar');
    
    checkedRows.forEach((item: IDictionary) => {
      item.OPERACAO = 'A';
      item.PRODUTOORIGEM = formFilter.CDPRODUTO.CDARVPROD;
      item.NUTRIENTESCONCAT = nutriSelected.join('_');
      item.PRODSELECIONADO = produtosSelected.join('_');
    });

    this.utilities.activeLoadingButton(button);
    Http.post('/nutriProdExport/acrescentar', { checkedRows }).then(() => {
      this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'green');
    }).finally(() => {
      DialogService.hide();
      this.utilities.disableLoadingButton(button);
      this.closeNutriModalExport();
    });
  }

  public sobrepor(checkedRows: IDictionary) {
    const formFilter = Metadata.getInstance<Form>('formFilter').value;
    const nutriente: GridEditable = Metadata.getInstance('NUTRIENTE');
    const rowNutriente = nutriente.selectedRows;
    const nutriSelected = rowNutriente.map((row: IDictionary) => row.CDNUTRIENTE);
    const nutrienteExport: Grid = Metadata.getInstance('gridExportNutrientes');
    const rowProdutos = nutrienteExport.selectedRows;
    const produtosSelected = rowProdutos.map((row: IDictionary) => row.CDARVPROD);
    const button = Metadata.getInstance<Button>('sobrepor');

    checkedRows.forEach((item: IDictionary) => {
      item.OPERACAO = 'S';
      item.PRODUTOORIGEM = formFilter.CDPRODUTO.CDARVPROD;
      item.NUTRIENTESCONCAT = nutriSelected.join('_');
      item.PRODSELECIONADO = produtosSelected.join('_');
    });

    this.utilities.activeLoadingButton(button);
    Http.post('/nutriProdExport/sobrepor', { checkedRows }).then(() => {
      this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'green');
    }).finally(() => {
      DialogService.hide();
      this.utilities.disableLoadingButton(button);
      this.closeNutriModalExport();
    });
  }

  public closeNutriModalExport() {
    this.modalExport.hide();
  }
}
